<?php
/**
 * Copyright (C) 2017-2019 thirty bees
 * Copyright (C) 2007-2016 PrestaShop SA
 *
 * thirty bees is an extension to the PrestaShop software by PrestaShop SA.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@thirtybees.com so we can send you a copy immediately.
 *
 * @author    thirty bees <modules@thirtybees.com>
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2017-2019 thirty bees
 * @copyright 2007-2016 PrestaShop SA
 * @license   Academic Free License (AFL 3.0)
 * PrestaShop is an internationally registered trademark of PrestaShop SA.
 */

if (!defined('_TB_VERSION_')) {
    exit;
}

/**
 * Class BlockViewed
 *
 * @since 1.0.0
 */
class BlockViewed extends Module
{

    /**
     * BlockViewed constructor.
     *
     * @since 1.0.0
     */
    public function __construct()
    {
        $this->name = 'blockviewed';
        $this->tab = 'front_office_features';
        $this->version = '2.0.1';
        $this->author = 'thirty bees';
        $this->need_instance = 0;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l('Block Viewed Products');
        $this->description = $this->l('Adds a block displaying recently viewed products.');
        $this->tb_versions_compliancy = '> 1.0.0';
        $this->tb_min_version = '1.0.0';
    }

    /**
     * @return bool
     *
     * @since 1.0.0
     */
    public function install()
    {

        if (!parent::install()
            || !$this->registerHook('header')
            || !$this->registerHook('leftColumn')
        ) {
            return false;
        }

        Configuration::updateValue('PRODUCTS_VIEWED_NBR', 2);

        return true;
    }

    /**
     * Called in administration -> module -> configure
     */
    public function getContent()
    {
        $output = '';
        if (Tools::isSubmit('submitBlockViewed')) {
            if (!($productNbr = Tools::getValue('PRODUCTS_VIEWED_NBR')) || empty($productNbr))
                $output .= $this->displayError($this->l('You must fill in the \'Products displayed\' field.'));
            elseif ((int)($productNbr) == 0)
                $output .= $this->displayError($this->l('Invalid number.'));
            else
            {
                Configuration::updateValue('PRODUCTS_VIEWED_NBR', (int)$productNbr);
                $output .= $this->displayConfirmation($this->l('Settings updated'));
            }
        }
        return $output.$this->renderForm();
    }

    /**
     * @param array $params
     *
     * @return bool|string
     *
     * @since 1.0.0
     */
    public function hookRightColumn($params)
    {
        $productsViewed = (isset($params['cookie']->viewed) && !empty($params['cookie']->viewed)) ? array_slice(array_reverse(explode(',', $params['cookie']->viewed)), 0, Configuration::get('PRODUCTS_VIEWED_NBR')) : array();

        if (count($productsViewed)) {
            $defaultCover = Language::getIsoById($params['cookie']->id_lang).'-default';

            $productIds = implode(',', array_map('intval', $productsViewed));
            $productsImages = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
                        SELECT MAX(image_shop.id_image) id_image, p.id_product, il.legend, product_shop.active, pl.name, pl.description_short, pl.link_rewrite, cl.link_rewrite AS category_rewrite
                        FROM '._DB_PREFIX_.'product p
                        '.Shop::addSqlAssociation('product', 'p').'
                        LEFT JOIN '._DB_PREFIX_.'product_lang pl ON (pl.id_product = p.id_product'.Shop::addSqlRestrictionOnLang('pl').')
                        LEFT JOIN '._DB_PREFIX_.'image i ON (i.id_product = p.id_product)'.
                        Shop::addSqlAssociation('image', 'i', false, 'image_shop.cover=1').'
                        LEFT JOIN '._DB_PREFIX_.'image_lang il ON (il.id_image = image_shop.id_image AND il.id_lang = '.(int)($params['cookie']->id_lang).')
                        LEFT JOIN '._DB_PREFIX_.'category_lang cl ON (cl.id_category = product_shop.id_category_default'.Shop::addSqlRestrictionOnLang('cl').')
                        WHERE p.id_product IN ('.$productIds.')
                        AND pl.id_lang = '.(int)($params['cookie']->id_lang).'
                        AND cl.id_lang = '.(int)($params['cookie']->id_lang).'
                        GROUP BY product_shop.id_product'
            );

            $productsImagesArray = array();
            foreach ($productsImages as $pi)
                $productsImagesArray[$pi['id_product']] = $pi;

            $productsViewedObj = array();
            foreach ($productsViewed as $productViewed)
            {
                $obj = (object)'Product';
                if (!isset($productsImagesArray[$productViewed]) || (!$obj->active = $productsImagesArray[$productViewed]['active']))
                    continue;
                else
                {
                    $obj->id = (int)($productsImagesArray[$productViewed]['id_product']);
                    $obj->id_image = (int)$productsImagesArray[$productViewed]['id_image'];
                    $obj->cover = (int)($productsImagesArray[$productViewed]['id_product']).'-'.(int)($productsImagesArray[$productViewed]['id_image']);
                    $obj->legend = $productsImagesArray[$productViewed]['legend'];
                    $obj->name = $productsImagesArray[$productViewed]['name'];
                    $obj->description_short = $productsImagesArray[$productViewed]['description_short'];
                    $obj->link_rewrite = $productsImagesArray[$productViewed]['link_rewrite'];
                    $obj->category_rewrite = $productsImagesArray[$productViewed]['category_rewrite'];
                    // $obj is not a real product so it cannot be used as argument for getProductLink()
                    $obj->product_link = $this->context->link->getProductLink($obj->id, $obj->link_rewrite, $obj->category_rewrite);

                    if (!isset($obj->cover) || !$productsImagesArray[$productViewed]['id_image']) {
                        $obj->cover = $defaultCover;
                        $obj->legend = '';
                    }
                    $productsViewedObj[] = $obj;
                }
            }

            if (!count($productsViewedObj))
                return;

            $this->smarty->assign(
                [
                    'productsViewedObj' => $productsViewedObj,
                    'mediumSize' => Image::getSize('medium'),
                ]
            );

            return $this->display(__FILE__, 'blockviewed.tpl');
        }
        return;
    }

    /**
     * @param array $params
     *
     * @return bool|string
     *
     * @since 1.0.0
     */
    public function hookLeftColumn($params)
    {
        return $this->hookRightColumn($params);
    }

    /**
     * @param array $params
     *
     * @return bool|string
     *
     * @since 1.0.0
     */
    public function hookFooter($params)
    {
        return $this->hookRightColumn($params);
    }

    /**
     * @param $params
     *
     * @since 1.0.0
     */
    public function hookHeader($params)
    {
        $id_product = (int)Tools::getValue('id_product');
        $productsViewed = (isset($params['cookie']->viewed) && !empty($params['cookie']->viewed)) ? array_slice(array_reverse(explode(',', $params['cookie']->viewed)), 0, Configuration::get('PRODUCTS_VIEWED_NBR')) : array();

        if ($id_product && !in_array($id_product, $productsViewed)) {
            $product = new Product((int)$id_product);
            if ($product->checkAccess((int)$this->context->customer->id)) {
                if (isset($params['cookie']->viewed) && !empty($params['cookie']->viewed))
                    $params['cookie']->viewed .= ','.(int)$id_product;
                else
                    $params['cookie']->viewed = (int)$id_product;
            }
        }
        $this->context->controller->addCSS($this->_path.'css/blockviewed.css', 'all');
    }

    /**
     * @return string
     *
     * @since 1.0.0
     */
    public function renderForm()
    {
        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Settings'),
                    'icon'  => 'icon-cogs',
                ],
                'input'  => [
                    [
                        'type'  => 'text',
                        'label' => $this->l('Products to display'),
                        'name'  => 'PRODUCTS_VIEWED_NBR',
                        'desc'  => $this->l('Define the number of products displayed in this block'),
                        'class' => 'fixed-width-xs',
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;

        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitBlockViewed';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false).'&configure='.$this->name.'&tab_module='.$this->tab.'&module_name='.$this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFieldsValues(),
            'languages'    => $this->context->controller->getLanguages(),
            'id_language'  => $this->context->language->id,
        ];

        return $helper->generateForm([$fields_form]);
    }

    /**
     * @return array
     *
     * @since 1.0.0
     */
    public function getConfigFieldsValues()
    {
        return [
            'PRODUCTS_VIEWED_NBR' => (int) Tools::getValue('PRODUCTS_VIEWED_NBR', Configuration::get('PRODUCTS_VIEWED_NBR')),
        ];
    }

}
