<?php
/**
 * Copyright (C) 2017-2019 thirty bees
 * Copyright (C) 2007-2016 PrestaShop SA
 *
 * thirty bees is an extension to the PrestaShop software by PrestaShop SA.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@thirtybees.com so we can send you a copy immediately.
 *
 * @author    thirty bees <modules@thirtybees.com>
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2017-2019 thirty bees
 * @copyright 2007-2016 PrestaShop SA
 * @license   Academic Free License (AFL 3.0)
 * PrestaShop is an internationally registered trademark of PrestaShop SA.
 */

require_once __DIR__.'/classes/autoload.php';

use TbUpdaterModule\Tools;

/**
 * Display SQL date in friendly format
 *
 * @param string  $sqlDate  Date in SQL format (YYYY-MM-DD HH:mm:ss)
 * @param boolean $withTime Display both date and time
 *
 * @todo Several formats (french : DD-MM-YYYY)
 */
function displayDate($sqlDate, $withTime = false)
{
    return strftime('%Y-%m-%d'.($withTime ? ' %H:%M:%S' : ''), strtotime($sqlDate));
}

/**
 * Return path to a product category
 *
 * @param string  $urlBase     Start URL
 * @param integer $idCategory Start category
 * @param string  $path        Current path
 * @param string  $highlight   String to highlight (in XHTML/CSS)
 * @param string  $type        Category type (products/cms)
 */
function getPath($urlBase, $idCategory, $path = '', $highlight = '', $categoryType = 'catalog')
{
    global $cookie;

    if ($categoryType == 'catalog') {
        $category = Db::getInstance()->getRow(
            '
		SELECT id_category, level_depth, nleft, nright
		FROM '._DB_PREFIX_.'category
		WHERE id_category = '.(int) $idCategory
        );

        if (isset($category['id_category'])) {
            $categories = Db::getInstance()->ExecuteS(
                '
			SELECT c.id_category, cl.name, cl.link_rewrite
			FROM '._DB_PREFIX_.'category c
			LEFT JOIN '._DB_PREFIX_.'category_lang cl ON (cl.id_category = c.id_category)
			WHERE c.nleft <= '.(int) $category['nleft'].' AND c.nright >= '.(int) $category['nright'].' AND cl.id_lang = '.(int) ($cookie->id_lang).'
			ORDER BY c.level_depth ASC
			LIMIT '.(int) ($category['level_depth'] + 1)
            );

            $fullPath = '';
            $n = 1;
            $nCategories = (int) sizeof($categories);
            foreach ($categories as $category) {
                $edit = '<a href="'.$urlBase.'&id_category='.(int) $category['id_category'].'&'.($category['id_category'] == 1 ? 'viewcategory' : 'addcategory').'&token='.Tools::getAdminToken('AdminCatalog'.(int) (Tab::getIdFromClassName('AdminCatalog')).(int) ($cookie->id_employee)).'" title="'.($category['id_category'] == 1 ? 'Home' : 'Modify').'"><img src="../img/admin/'.($category['id_category'] == 1 ? 'home' : 'edit').'.gif" alt="" /></a> ';
                $fullPath .= $edit.
                    ($n < $nCategories ? '<a href="'.$urlBase.'&id_category='.(int) $category['id_category'].'&viewcategory&token='.Tools::getAdminToken('AdminCatalog'.(int) (Tab::getIdFromClassName('AdminCatalog')).(int) ($cookie->id_employee)).'" title="'.htmlentities($category['name'], ENT_NOQUOTES, 'UTF-8').'">' : '').
                    (!empty($highlight) ? str_ireplace($highlight, '<span class="highlight">'.htmlentities($highlight, ENT_NOQUOTES, 'UTF-8').'</span>', $category['name']) : $category['name']).
                    ($n < $nCategories ? '</a>' : '').
                    (($n++ != $nCategories || !empty($path)) ? ' > ' : '');
            }

            return $fullPath.$path;
        }
    } elseif ($categoryType == 'cms') {
        $category = new CMSCategory($idCategory, (int) ($cookie->id_lang));
        if (!$category->id) {
            return $path;
        }

        $name = ($highlight != null) ? str_ireplace($highlight, '<span class="highlight">'.$highlight.'</span>', CMSCategory::hideCMSCategoryPosition($category->name)) : CMSCategory::hideCMSCategoryPosition($category->name);
        $edit = '<a href="'.$urlBase.'&id_cms_category='.$category->id.'&addcategory&token='.Tools::getAdminToken('AdminCMSContent'.(int) (Tab::getIdFromClassName('AdminCMSContent')).(int) ($cookie->id_employee)).'">
				<img src="../img/admin/edit.gif" alt="Modify" /></a> ';
        if ($category->id == 1) {
            $edit = '<a href="'.$urlBase.'&id_cms_category='.$category->id.'&viewcategory&token='.Tools::getAdminToken('AdminCMSContent'.(int) (Tab::getIdFromClassName('AdminCMSContent')).(int) ($cookie->id_employee)).'">
					<img src="../img/admin/home.gif" alt="Home" /></a> ';
        }
        $path = $edit.'<a href="'.$urlBase.'&id_cms_category='.$category->id.'&viewcategory&token='.Tools::getAdminToken('AdminCMSContent'.(int) (Tab::getIdFromClassName('AdminCMSContent')).(int) ($cookie->id_employee)).'">
		'.$name.'</a> > '.$path;
        if ($category->id == 1) {
            return substr($path, 0, strlen($path) - 3);
        }

        return getPath($urlBase, $category->id_parent, $path, '', 'cms');
    }
}

function getDirContent($path)
{
    $content = [];
    if (is_dir($path)) {
        $d = dir($path);
        while (false !== ($entry = $d->read())) {
            if ($entry{0} != '.') {
                $content[] = $entry;
            }
        }
        $d->close();
    }

    return $content;
}

function createDir($path, $rights)
{
    if (file_exists($path)) {
        return true;
    }

    return @mkdir($path, $rights);
}

function recursiveTab($idTab)
{
    global $cookie, $tabs;

    $adminTab = Tab::getTab((int) $cookie->id_lang, $idTab);
    $tabs[] = $adminTab;
    if ($adminTab['id_parent'] > 0) {
        recursiveTab($adminTab['id_parent']);
    }
}

function checkingTab($tab)
{
    global $ajaxUpgrader, $cookie;

    $tab = trim($tab);
    if (!Validate::isTabName($tab)) {
        return false;
    }

    $row = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow($sql = 'SELECT id_tab, module FROM `'._DB_PREFIX_.'tab` WHERE class_name = \''.pSQL($tab).'\'');
    if (!$row['id_tab']) {
        if (isset(\AdminTab::$tabParenting[$tab])) {
            Tools::redirectAdmin('?tab='.\AdminTab::$tabParenting[$tab].'&token='.Tools::getAdminTokenLite(\AdminTab::$tabParenting[$tab]));
        }
        echo Tools::displayError('Tab cannot be found.');

        return false;
    }
    if ($row['module'] AND file_exists(_PS_MODULE_DIR_.'/'.$row['module'].'/'.$tab.'.php')) {
        include_once(_PS_MODULE_DIR_.'/'.$row['module'].'/'.$tab.'.php');
    } elseif (file_exists(PS_ADMIN_DIR.'/tabs/'.$tab.'.php')) {
        include_once(PS_ADMIN_DIR.'/tabs/'.$tab.'.php');
    }

    if (!class_exists($tab, false) OR !$row['id_tab']) {
        echo Tools::displayError('Tab file cannot be found.');

        return false;
    }
    $ajaxUpgrader = new $tab;
    if (!$ajaxUpgrader->viewAccess()) {
        $ajaxUpgrader->_errors = [Tools::displayError('Access denied')];
        echo $ajaxUpgrader->displayErrors();

        return false;
    }

    return $row['id_tab'];
}

function checkTabRights($idTab)
{
    global $cookie;
    static $tabAccesses = null;

    if ($tabAccesses === null) {
        $tabAccesses = Profile::getProfileAccesses($cookie->profile);
    }

    if (isset($tabAccesses[(int) ($idTab)]['view'])) {
        return ($tabAccesses[(int) ($idTab)]['view'] === '1');
    }

    return false;
}

/**
 * Converts a simpleXML element into an array. Preserves attributes and everything.
 * You can choose to get your elements either flattened, or stored in a custom index that
 * you define.
 * For example, for a given element
 * <field name="someName" type="someType"/>
 * if you choose to flatten attributes, you would get:
 * $array['field']['name'] = 'someName';
 * $array['field']['type'] = 'someType';
 * If you choose not to flatten, you get:
 * $array['field']['@attributes']['name'] = 'someName';
 * _____________________________________
 * Repeating fields are stored in indexed arrays. so for a markup such as:
 * <parent>
 * <child>a</child>
 * <child>b</child>
 * <child>c</child>
 * </parent>
 * you array would be:
 * $array['parent']['child'][0] = 'a';
 * $array['parent']['child'][1] = 'b';
 * ...And so on.
 * _____________________________________
 *
 * @param simpleXMLElement $xml               the XML to convert
 * @param boolean          $flattenValues     Choose wether to flatten values
 *                                            or to set them under a particular index.
 *                                            defaults to true;
 * @param boolean          $flattenAttributes Choose wether to flatten attributes
 *                                            or to set them under a particular index.
 *                                            Defaults to true;
 * @param boolean          $flattenChildren   Choose wether to flatten children
 *                                            or to set them under a particular index.
 *                                            Defaults to true;
 * @param string           $valueKey          index for values, in case $flattenValues was set to
 *                                            false. Defaults to "@value"
 * @param string           $attributesKey     index for attributes, in case $flattenAttributes was set to
 *                                            false. Defaults to "@attributes"
 * @param string           $childrenKey       index for children, in case $flattenChildren was set to
 *                                            false. Defaults to "@children"
 *
 * @return array the resulting array.
 */
function simpleXMLToArray($xml, $flattenValues = true, $flattenAttributes = true, $flattenChildren = true, $valueKey = '@value', $attributesKey = '@attributes', $childrenKey = '@children')
{
    $return = [];
    if (!($xml instanceof SimpleXMLElement)) {
        return $return;
    }

    $name = $xml->getName();
    $trimmedValue = trim((string) $xml);
    if (strlen($trimmedValue) == 0) {
        $trimmedValue = null;
    }

    if ($trimmedValue !== null) {
        if (!$flattenValues) {
            $return[$valueKey] = $trimmedValue;
        } else {
            $return = $trimmedValue;
        }
    }

    $children = [];
    $first = true;
    foreach ($xml->children() as $elementName => $child) {
        $value = simpleXMLToArray($child, $flattenValues, $flattenAttributes, $flattenChildren, $valueKey, $attributesKey, $childrenKey);
        if (isset($children[$elementName])) {
            if ($first) {
                $temp = $children[$elementName];
                unset($children[$elementName]);
                $children[$elementName][] = $temp;
                $first = false;
            }
            $children[$elementName][] = $value;
        } else {
            $children[$elementName] = $value;
        }
    }

    if (count($children) > 0) {
        if (!$flattenChildren) {
            $return[$childrenKey] = $children;
        } else {
            $return = array_merge($return, $children);
        }
    }

    $attributes = [];
    foreach ($xml->attributes() as $name => $value) {
        $attributes[$name] = trim($value);
    }

    if (count($attributes) > 0) {
        if (!$flattenAttributes) {
            $return[$attributesKey] = $attributes;
        } else {
            $return = array_merge($return, $attributes);
        }
    }

    return $return;
}
